from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class MembershipType(str, Enum):
    """
    MembershipType
    """

    """
    allowed enum values
    """
    MEMBER = "MEMBER"
    ADMIN = "ADMIN"
    NON_DELETE_ADMIN = "NON_DELETE_ADMIN"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of MembershipType from a JSON string"""
        return cls(json.loads(json_str))
